/*
Copyright (C) 2002 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of JIM - Java Information Manager
(http://www.kclee.com/clemens/java/jim/).

Jim is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jim is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jim; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jim;

import ccl.swing.AutoGridBagLayout;
import ccl.swing.DateField;
import ccl.swing.MainJFrame;
import ccl.swing.OKCancelPanel;
import ccl.swing.SwingUtil;
import ccl.util.Util;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.border.Border;
import javax.swing.table.TableColumn;

/**
 * 
 * @author  Chr. Clemens Lee
 * @version $Id: AnniversariesPanel.java,v 1.5 2002/04/21 13:25:16 clemens Exp clemens $
 */
public class AnniversariesPanel extends JPanel {
    private static final String S_MAIN = "MAIN";
    private static final String S_EDIT = "EDIT";

    private JButton _btnAdd = null;
    private JButton _btnEdit = null;
    private JButton _btnDelete = null;
    private Vector _vAnniversaries = null;
    private AnniversariesTableModel _model = null;
    private JPanel _pnlMain = null;
    private JPanel _pnlEdit = null;
    private CardLayout _pCardLayout = null;
    private JTextField _txtName = null;
    private OKCancelPanel _pnlOKCancelEdit = null;
    private JList _lstAnniversaries = null;
    private DateField _txtDate = null;
    private JTable    _table   = null;

    private Anniversary _anEdit = null;

    private MainJFrame _frmMain = null;

    public AnniversariesPanel( MainJFrame frmMain_ ) {
        super();

        _frmMain = frmMain_;

        _vAnniversaries = new Vector();
        _model = new AnniversariesTableModel();
        _model.setAnniversaries( _vAnniversaries );

        _btnAdd = new JButton( "New" );
        _btnAdd.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent pActionEvent_ ) {                
                _anEdit =
                       new Anniversary();
                _txtName.setText( _anEdit.getName() );
                _txtDate.setText( _anEdit.getDate() );
                _pCardLayout.show( AnniversariesPanel.this,
                                   S_EDIT );
            }
        } );        
        _btnEdit = new JButton( "Edit" );
        _btnEdit.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent pActionEvent_ ) {                
                int selection = _table.getSelectedRow();
                if ( selection == -1 ) {
                    SwingUtil.showMessage
                           ( _frmMain, "Nothing was selected, action dismissed!" );

                    return;
                }
                _anEdit = (Anniversary)_vAnniversaries.
                       elementAt( selection );
                _model.removeAnniversary( selection );
                _txtName.setText( _anEdit.getName() );
                _txtDate.setText( _anEdit.getDate() );
                _pCardLayout.show( AnniversariesPanel.this,
                                   S_EDIT );
            }
        } );        
        _btnDelete = new JButton( "Delete" );
        _btnDelete.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent pActionEvent_ ) {                
                int selection = _table.getSelectedRow();
                if ( selection == -1 ) {
                    SwingUtil.showMessage
                           ( _frmMain, "Nothing was selected, action dismissed!" );

                    return;
                }
                _model.removeAnniversary( selection );
            }
        } );        

        _pnlMain = new JPanel( new BorderLayout() );
        _pnlMain.setBorder( BorderFactory.createEmptyBorder( 7, 7, 7, 7 ) );

        _table = new JTable( _model );
        _table.setSelectionMode
               ( ListSelectionModel.SINGLE_SELECTION );

        _table.setBackground( getBackground() );
      
        TableColumn tbcExec = _table.getColumnModel().getColumn( 0 );
        tbcExec.setWidth( 100 );
        tbcExec.setMinWidth( 100 );
        tbcExec.setMaxWidth( 100 );
        tbcExec.setPreferredWidth( 100 );

        _table.setPreferredScrollableViewportSize( new Dimension( 300, 300 ) );

        JScrollPane tableScrollpane = new JScrollPane( _table );


        _lstAnniversaries = new JList( new DefaultListModel() );
        JScrollPane scpAnniversaries = new JScrollPane( _lstAnniversaries );
        scpAnniversaries.setBorder
               ( SwingUtil.createCCLBorder() );

        AutoGridBagLayout mainLayout = new AutoGridBagLayout();
        mainLayout.setLayoutOn( _pnlMain );
        mainLayout.setExpandBoth();
        mainLayout.setFillBoth();
        mainLayout.setExtend( 1, 4 );
        mainLayout.add( _pnlMain, tableScrollpane );
        mainLayout.setExtend( 1, 1 );
        mainLayout.setExpandNone();
        mainLayout.setFillNone();
        mainLayout.add( _pnlMain, Box.createHorizontalStrut
                                      ( 5 ) );
        mainLayout.setAnchor( AutoGridBagLayout.NORTH );
        mainLayout.setFillHorizontal();
        mainLayout.add( _pnlMain, _btnAdd );
        mainLayout.nextLine();
        mainLayout.add( _pnlMain, _btnEdit );
        mainLayout.nextLine();
        mainLayout.add( _pnlMain, _btnDelete );
        mainLayout.nextLine();
        mainLayout.setFillNone();
        mainLayout.setExpandVertical();
        mainLayout.add( _pnlMain, new JLabel( " " ) );
        
        AutoGridBagLayout agblEdit = new AutoGridBagLayout();
        _pnlEdit = new JPanel( agblEdit );
        _pnlEdit.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );

        _txtName = new JTextField();
        _txtName.setBorder( SwingUtil.createCCLBorder() );
        _pnlEdit.add( new JLabel( "Name: " ) );
        agblEdit.setFillHorizontal();
        agblEdit.setExpandHorizontal();
        _pnlEdit.add( _txtName );
        agblEdit.setExpandNone();
        agblEdit.setFillNone();
        _pnlEdit.add( new JLabel( " " ) );
        _pnlOKCancelEdit = new OKCancelPanel( new JDialog(), true );
        agblEdit.setAnchor( GridBagConstraints.EAST );
        agblEdit.setExtend( 1, 2 );
        _pnlEdit.add( _pnlOKCancelEdit );
        agblEdit.setExtend( 1, 1 );
        agblEdit.endLine();
        
        agblEdit.setAnchor( GridBagConstraints.WEST );
        _pnlEdit.add( new JLabel( "Date: " ) );
        _txtDate = new DateField( Util.getDate( Util.getCalendar() ) );
        _txtDate.setBorder( SwingUtil.createCCLBorder() );
        _pnlEdit.add( _txtDate );
        agblEdit.setExpandNone();
        agblEdit.setFillNone();

        _pnlOKCancelEdit.getCancelButton().
               addActionListener( new ActionListener() {
                   public void actionPerformed( ActionEvent pActionEvent_ ) {
                       _pCardLayout.show( AnniversariesPanel.this, S_MAIN );
                       if ( !_anEdit.isEmpty() ) {
                           _model.addAnniversary( _anEdit );
                       }
                   }
               } );
        _pnlOKCancelEdit.getOKButton().
               addActionListener( new ActionListener() {
                   public void actionPerformed( ActionEvent pActionEvent_ ) {
                       String sName = _txtName.getText().trim();
                       String sDate = _txtDate.getText();
                       if ( sName.equals( "" ) || sDate.equals( "" ) ) {
                           SwingUtil.showMessage
                                  ( _frmMain, "Either date of birth or name was not given. Nothing gets saved!" );

                           return;
                       }
                       _frmMain.setModified( true );
                       _anEdit.setName( sName );
                       _anEdit.setDate( sDate );
                       _pCardLayout.show( AnniversariesPanel.this, S_MAIN );
                       _model.addAnniversary( _anEdit );
                   }
               } );
        

        _pCardLayout = new CardLayout();

        setLayout( _pCardLayout );
        add( _pnlMain, S_MAIN );
        add( _pnlEdit, S_EDIT );
        _pCardLayout.show( this, S_MAIN );
    }

    public Vector getAnniversaries() {
        return _vAnniversaries;
    }
 
    public String getName() {
        return "Anniversaries";
    }

    public void setAnniversaries( Vector vAnniversaries_ ) {
        _vAnniversaries = vAnniversaries_;

        _model.setAnniversaries( _vAnniversaries );

        DefaultListModel dlmAnniversaries = 
               (DefaultListModel)_lstAnniversaries.getModel();
        _lstAnniversaries.clearSelection();
        dlmAnniversaries.removeAllElements();
        Enumeration eAnniversaries = _vAnniversaries.elements();
        while( eAnniversaries.hasMoreElements() ) {
            dlmAnniversaries.addElement( eAnniversaries.nextElement() );
        }
    }
}
