/*
Copyright (C) 2002 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of JIM - Java Information Manager
(http://www.kclee.com/clemens/java/jim/).

Jim is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jim is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jim; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jim;

import ccl.util.Util;
import java.util.Vector;
import javax.swing.table.AbstractTableModel;

/**
 * @author  Chr. Clemens Lee
 * @version $Id: AnniversariesTableModel.java,v 1.3 2002/04/21 13:25:19 clemens Exp clemens $
 */
public class AnniversariesTableModel extends AbstractTableModel {
    private static final String[] AS_COLUMN = 
        { "Date of Birth", "Name" };
    private Vector _vAnniversaries = null;

    public AnniversariesTableModel() {
        super();

        _vAnniversaries = new Vector();
    }

    public void setAnniversaries( Vector vAnniversaries_ ) {
        Util.debug( this, "setAnniversaries(..).ENTER" );
        int oldRows = _vAnniversaries.size();

        if ( oldRows > vAnniversaries_.size() ) {
            fireTableRowsDeleted( vAnniversaries_.size(), oldRows - 1 );
        }

        _vAnniversaries = vAnniversaries_;
        Util.sortFast( _vAnniversaries );

        fireTableRowsUpdated
               ( 0, Math.min( _vAnniversaries.size(), oldRows ) - 1 );

        if ( _vAnniversaries.size() > oldRows ) {
            fireTableRowsInserted( oldRows, _vAnniversaries.size() -1 );
        }
    }

    public void addAnniversary( Anniversary anniversary_ ) {
        fireTableRowsDeleted( 0, getRowCount() );
        _vAnniversaries.addElement( anniversary_ );
        Util.sortFast( _vAnniversaries );
        fireTableRowsInserted( 0, getRowCount() );
    }

    public void removeAnniversary( int row_ ) {
        _vAnniversaries.removeElementAt( row_ );
        fireTableRowsDeleted( row_, row_ );
    }

    public int getColumnCount() {
        return AS_COLUMN.length;
    }

    public int getRowCount() {
        return _vAnniversaries.size();
    }

    public String getColumnName( int column_ ) {
        return AS_COLUMN[ column_ ];
    }

    public Object getValueAt( int row_, int column_ ) {
        Util.debug( this, "getValueAt(..).row_: " +
                    row_ );
        Anniversary anniversary = (Anniversary)_vAnniversaries.elementAt( row_ );

        if ( column_ == 0 ) {
            return anniversary.getDate();
        }
        if ( column_ == 1 ) {
            return anniversary.getName();
        }

        return new Object();
    }

    public void setValueAt( Object oValue_, 
                            int row_, 
                            int column_ )
    {
        Util.panicIf( true, "This method should not be used yet, use setAnniversaries(..) intead." );

        fireTableRowsUpdated( row_, row_ );
    }

    public boolean isCellEditable( int row_, int column_ ) {
        return false;
    }

    public Class getColumnClass( int column_ ) {
        return super.getColumnClass( column_ );
    }

    public static String[] getColumnNames() {
        return AS_COLUMN;
    }
}
