/*
Copyright (C) 2002 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of JIM - Java Information Manager
(http://www.kclee.com/clemens/java/jim/).

Jim is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jim is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jim; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jim;

import ccl.swing.AutoGridBagLayout;
import ccl.swing.OKCancelPanel;
import ccl.swing.SwingUtil;
import ccl.util.Util;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.text.JTextComponent;
import javax.swing.text.Keymap;
import javax.swing.text.TextAction;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

/**
 * 
 * @author  Chr. Clemens Lee
 * @version $Id: NotesPanel.java,v 1.6 2002/04/21 13:25:23 clemens Exp clemens $
 */
public class NotesPanel extends JPanel 
                        implements TreeSelectionListener
{
    private JTextArea _txtNotes = null;
    private ViewController _pViewController = null;

    /**
     * We want to rely on the fact that always one item is
     * selected in the tree. Also, always at least one node
     * has to be present in the tree.
     */
    private JTree _trNotes = null;
    private DefaultMutableTreeNode _tnRoot = null;
    private DefaultMutableTreeNode _ndLast = null;
    private JButton _btnNewNote = null;
    private JButton _btnCutNote = null;
    private JButton _btnPasteNote = null;
    private JTextField _txtInput = null;
    private OKCancelPanel _pOKCancelPanel = null;

    private static final JDialog DLG_DUMMY = new JDialog();
    private WindowAdapter _waOKCancelPanel = null;
    private boolean _bInputVisible = false;
    private boolean _bInputVisibleBefore = false;

    private void _setInputVisible( boolean bVisible_ ) {
        if ( _bInputVisible == bVisible_ ) {
            // nothing to do
            return;
        }

        /*_waOKCancelPanel.windowClosing( null );*/
        _txtInput.setVisible( bVisible_ );
        _pOKCancelPanel.setVisible( bVisible_ );
        _txtInput.invalidate();
        _pOKCancelPanel.invalidate();
        validate();
        _pOKCancelPanel.validate();

        if ( bVisible_ ) {
            _txtInput.requestFocus();
            _waOKCancelPanel.windowOpened( null );
            _bInputVisibleBefore = true;
        } else {
            if ( _bInputVisibleBefore ) {
                _waOKCancelPanel.windowClosed( null );
                _bInputVisibleBefore = false;
            }
        }

        if ( _trNotes != null && _txtInput != null ) {
            _trNotes.setEnabled( !bVisible_ );
            _txtNotes.setEnabled( !bVisible_ );
        }

        _bInputVisible = bVisible_;
    }

    private DefaultMutableTreeNode _tnCutNote = null;

    public DefaultMutableTreeNode getSelectedNode() {
        TreePath pTreePath = _trNotes.getSelectionPath();
        DefaultMutableTreeNode tnSelected = 
               (DefaultMutableTreeNode)
               pTreePath.getLastPathComponent();
        
        return tnSelected;
    }

    public void deleteNote() {
        // might not be necessary, but can't hurt
        saveLastNode();

        // get selected note
        TreePath pTreePath = _trNotes.getSelectionPath();
        DefaultMutableTreeNode tnSelected = 
               (DefaultMutableTreeNode)
               pTreePath.getLastPathComponent();

        // don't delete top node
        DefaultMutableTreeNode tnParent = (DefaultMutableTreeNode)tnSelected.getParent();
        if ( tnParent == null ) {
            SwingUtil.showMessage( _pViewController, "Sorry, you can't delete this note!" );
            
            return;
        }

        // sure to delete this note?
        boolean bDelete = SwingUtil.isOKOrCancel
               ( _pViewController, 
                 "Is it OK to delete note\n'" +
                 tnSelected.toString() +
                 "'\nand all of its sub notes?" );

        _tnCutNote = tnSelected;

        // delete it
        if ( bDelete ) {
            DefaultTreeModel pModel = (DefaultTreeModel)_trNotes.getModel();
            TreePath tpParent = pTreePath.getParentPath();
            pModel.removeNodeFromParent( tnSelected );
            
            // make sure something is selected
            _trNotes.setSelectionPath( tpParent );
        }

        _pViewController.setModified( true );
        _btnPasteNote.setEnabled( true );
    }

    public void pasteNote() {
        saveLastNode();

        if ( _tnCutNote == null ) {
            // nothing to paste
            return;
        }

        // find parent
        TreePath pTreePath = _trNotes.getSelectionPath();
        DefaultMutableTreeNode tnParent = (DefaultMutableTreeNode)
               pTreePath.getLastPathComponent();

        // find new index
        Note pNote = (Note)_tnCutNote.getUserObject();
        int newIndex = 0;
        Enumeration eChildren = tnParent.children();
        while( eChildren.hasMoreElements() ) {
            DefaultMutableTreeNode tnNext = (DefaultMutableTreeNode)
                   eChildren.nextElement();
            if ( pNote.toString().compareTo
                 ( tnNext.getUserObject().toString() ) < 0 )
            {
                break;
            }
            newIndex++;
        }

        // insert
        DefaultMutableTreeNode tnNew =
               cloneNoteNode( _tnCutNote );
        ((DefaultTreeModel)_trNotes.getModel()).
               insertNodeInto( tnNew, tnParent, newIndex );

        _pViewController.setModified( true );
    }

    public void createNewNote( String sKey_ ) {
        saveLastNode();

        // create new node
        Note pNote = new Note( sKey_ );
        //TreeNode tnNew = new DefaultMutableTreeNode( pNote );
        
        // find parent
        TreePath pTreePath = _trNotes.getSelectionPath();
        DefaultMutableTreeNode tnParent = (DefaultMutableTreeNode)
               pTreePath.getLastPathComponent();

        // find new index
        int newIndex = 0;
        Enumeration eChildren = tnParent.children();
        while( eChildren.hasMoreElements() ) {
            DefaultMutableTreeNode tnNext = (DefaultMutableTreeNode)
                   eChildren.nextElement();
            if ( pNote.toString().compareTo
                 ( tnNext.getUserObject().toString() ) < 0 )
            {
                break;
            }
            newIndex++;
        }

        // insert
        DefaultMutableTreeNode tnNew =
               new DefaultMutableTreeNode( pNote );
        ((DefaultTreeModel)_trNotes.getModel()).
               insertNodeInto( tnNew, tnParent, newIndex );

        _pViewController.setModified( true );
    }

    /**
     * Maybe it should be named, saveCurrentNode().
     */
    public synchronized void saveLastNode() {
        if ( _ndLast != null ) {
            Note pNote = (Note)_ndLast.getUserObject();
            pNote.setNote( _txtNotes.getText() );
        }
    }

    public void insertDate() {
        if ( !_txtNotes.isEnabled() ) {
            return;
        }
        SwingUtil.invokeLaterIfNecessary( new Runnable() {
            public void run() {
                Date pDate = new Date();
                SimpleDateFormat pDateFormat
                       = new SimpleDateFormat ("yyyy-MM-dd");
                _txtNotes.replaceSelection( pDateFormat.format( pDate ) );
                }
        } );
    }

    public class DateTextAction extends TextAction {
        public DateTextAction() {
            super( "DateTextAction" );
        }

        public void actionPerformed( ActionEvent pActionEvent_ ) {
            JTextComponent txtTarget = getTextComponent( pActionEvent_ );
            if ( txtTarget.getName() != null &&
                 txtTarget.getName().equals( "_txtNotes" ) ) 
            {
                insertDate();
            }
        }
    }

    public NotesPanel( ViewController pViewController_ ) {
        super();

        _pViewController = pViewController_;

        AutoGridBagLayout pAutoGridBagLayout = 
               new AutoGridBagLayout();

        setLayout( pAutoGridBagLayout );

        pAutoGridBagLayout.setAnchor( GridBagConstraints.WEST );

        _btnNewNote = new JButton( "New Note" );
	_btnNewNote.setName( "AddButton" );
        _btnNewNote.addActionListener
               ( new ActionListener() {
                   public void actionPerformed( ActionEvent pActionEvent_ ) {
                       _setInputVisible( true );
                   }
               } );
        add( _btnNewNote );

        _btnCutNote = new JButton( "Cut Note" );
	_btnCutNote.setName( "DeleteButton" );
        _btnCutNote.addActionListener
               ( new ActionListener() {
                   public void actionPerformed( ActionEvent pActionEvent_ ) {
                       _setInputVisible( false );
                       deleteNote();
                   }
               } );
        add( _btnCutNote );

        _btnPasteNote = new JButton( "Paste Note" );
	_btnPasteNote.setName( "PasteButton" );
        _btnPasteNote.setEnabled( false );
        _btnPasteNote.addActionListener
               ( new ActionListener() {
                   public void actionPerformed( ActionEvent pActionEvent_ ) {
                       _setInputVisible( false );
                       pasteNote();
                   }
               } );
        add( _btnPasteNote );

        add( new JLabel( " " ) );

        _txtInput = new JTextField();
	_txtInput.setName( "InputText" );
        _txtInput.setBorder( SwingUtil.createCCLBorder() );
        pAutoGridBagLayout.setExpandHorizontal();
        pAutoGridBagLayout.setFillHorizontal();
        add( _txtInput );
        pAutoGridBagLayout.setExpandNone();
        pAutoGridBagLayout.setFillNone();

        _pOKCancelPanel = new OKCancelPanel( DLG_DUMMY, 0 );
        JButton btnOK = _pOKCancelPanel.getOKButton();
        _pViewController.getRootPane().setDefaultButton( btnOK );
        _waOKCancelPanel = _pOKCancelPanel.getWindowAdapter();
        DLG_DUMMY.removeWindowListener( _waOKCancelPanel );
        add( _pOKCancelPanel );

        // true necessarc or set visible will have no effect
        _bInputVisible = true;
        _setInputVisible( false );

        _pOKCancelPanel.getOKButton().addActionListener
               ( new ActionListener() {
                   public void actionPerformed( ActionEvent pActionEvent_ ) {
                       String sInput = _txtInput.getText().trim();
                       createNewNote( sInput );
                       _setInputVisible( false );
                   }
               } );
        _pOKCancelPanel.getCancelButton().addActionListener
               ( new ActionListener() {
                   public void actionPerformed( ActionEvent pActionEvent_ ) {
                       _setInputVisible( false );
                   }
               } );
        pAutoGridBagLayout.endLine();

        _txtNotes = new JTextArea();
        _txtNotes.setFont( new Font( "Monospaced", Font.PLAIN, 12 ) ); 
        JScrollPane scpText = new JScrollPane( _txtNotes );
        scpText.setBorder( SwingUtil.createCCLBorder() );

        DocumentListener pDocumentListener = new DocumentListener() {
            public void insertUpdate(DocumentEvent e) {
                if ( _bValueChanged ) {
                    return;
                }
                _pViewController.stateChanged( null );
            }

            public void removeUpdate(DocumentEvent e) {
                if ( _bValueChanged ) {
                    return;
                }
                _pViewController.stateChanged( null );
            }

            public void changedUpdate(DocumentEvent e) {
                if ( _bValueChanged ) {
                    return;
                }
                _pViewController.stateChanged( null );
            }
        };
        _txtNotes.getDocument().addDocumentListener( pDocumentListener );
        _txtNotes.setName( "_txtNotes" );

        // set date insertion text action
        TextAction pDateTextAction = new DateTextAction();
        Keymap kmOld = _txtNotes.getKeymap();
        Keymap kmNew = _txtNotes.addKeymap( null, kmOld );
        KeyStroke ksCtrlD = KeyStroke.getKeyStroke
               ( KeyEvent.VK_D, Event.CTRL_MASK );
        kmNew.removeKeyStrokeBinding( ksCtrlD );
        kmNew.addActionForKeyStroke
               ( ksCtrlD, pDateTextAction);
        _txtNotes.setKeymap( kmNew );

        _tnRoot = createRootNode();
        _trNotes = new JTree( _tnRoot );
        _trNotes.setName( "NotesTree" );
        _trNotes.getSelectionModel().
               setSelectionMode( TreeSelectionModel.SINGLE_TREE_SELECTION );
        JScrollPane scpTree = new JScrollPane( _trNotes );
        scpTree.setBorder( SwingUtil.createCCLBorder() );

        _trNotes.addTreeSelectionListener( this );
        _trNotes.setSelectionInterval( 0, 0 );

        JSplitPane pSplitPane = new JSplitPane();
        pSplitPane.setBorder( BorderFactory.createEmptyBorder( 0, 7, 7, 7 ) );
        pSplitPane.setLeftComponent( scpTree );
        pSplitPane.setRightComponent( scpText );
        pSplitPane.setDividerSize( 2 );
        pSplitPane.setDividerLocation( 300 );
        Util.debug( this, "<init>(..).splitpane.size: " +
                    pSplitPane.getSize() );
        Util.debug( this, "<init>(..).splitpane.dividerLocation: " +
                    pSplitPane.getDividerLocation() );

        pAutoGridBagLayout.setExtend( 7, 1 );
        pAutoGridBagLayout.setExpandBoth();
        pAutoGridBagLayout.setFillBoth();

        add( pSplitPane );
    }

    private boolean _bValueChanged = false;

    public synchronized void valueChanged( TreeSelectionEvent pTreeSelectionEvent_ ) {        
        Util.debug( "NotesPanel.valueChanged(..).START" );
        //TreePath pTreePath = pTreeSelectionEvent_.getNewLeadSelectionPath();
        saveLastNode();
        DefaultMutableTreeNode pNode = (DefaultMutableTreeNode)_trNotes.
               getLastSelectedPathComponent();
        if ( pNode == null ) {
            return;
        }
        _bValueChanged = true;
        _txtNotes.setText( ((Note)pNode.getUserObject()).getNote() );
        _bValueChanged = false;
        _ndLast = pNode;
    }

    public String getName() {
        return getNotesName();
    }

    public DefaultMutableTreeNode getNotes() {
        saveLastNode();
        
        return _tnRoot;
    }

    public void setNotes( DefaultMutableTreeNode tnRoot_ ) {
        _ndLast = null;
        _tnRoot = tnRoot_;
        _trNotes.setModel( new DefaultTreeModel( _tnRoot ) );
        _trNotes.setSelectionInterval( 0, 0 );
    }

    public static DefaultMutableTreeNode createRootNode() {
        Note ntRoot = new Note( getNotesName() );
        DefaultMutableTreeNode tnRoot = new DefaultMutableTreeNode( ntRoot );

        return tnRoot;
    }

    public static final String getNotesName() {
        return "Notes";
    }

    public static DefaultMutableTreeNode cloneNoteNode( DefaultMutableTreeNode tnNote_ ) {
        DefaultMutableTreeNode tnRetVal = null;

        if ( tnNote_ == null ) {
            return tnRetVal;
        }

        Note pNote = (Note)((Note)tnNote_.getUserObject()).clone();
        tnRetVal = new DefaultMutableTreeNode( pNote );

        // go through children
        for ( int child = 0; child < tnNote_.getChildCount(); child++ ) {
            DefaultMutableTreeNode tnNextChild = (DefaultMutableTreeNode)tnNote_.getChildAt( child );
            DefaultMutableTreeNode tnNextClone = cloneNoteNode( tnNextChild );
            
            // add child to return value
            tnRetVal.add( tnNextClone );
        }

        return tnRetVal;
    }
}
