/*
Copyright (C) 2002 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of JIM - Java Information Manager
(http://www.kclee.com/clemens/java/jim/).

Jim is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jim is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jim; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jim;

import ccl.swing.AutoGridBagLayout;
import ccl.swing.CCLBorder;
import ccl.swing.DateField;
import ccl.swing.OKCancelPanel;
import ccl.swing.SwingUtil;
import ccl.util.Comparable;
import ccl.util.Util;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.border.BevelBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.text.JTextComponent;
import javax.swing.text.Keymap;
import javax.swing.text.TextAction;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

/**
 * The panel for all the ToDo stuff.
 *
 * @author  Chr. Clemens Lee
 * @version $Id: ToDoPanel.java,v 1.7 2002/04/21 13:25:24 clemens Exp clemens $
 */
public class ToDoPanel extends JPanel 
                       implements TreeSelectionListener
                                  , ListSelectionListener
                                  , ChangeListener
{
    private static final int STATE_STARTUP = 0;
    private static final int STATE_NORMAL  = STATE_STARTUP + 1;
    private static final int STATE_INPUT   = STATE_NORMAL  + 1;
    private static final int STATE_TASK    = STATE_INPUT   + 1;

    private static String S_LIST_NAME = "ToDoListPane";

    /**
     * Used by jim test.
     */
    public static String S_TASK_NAME = "ToDoTaskPane";

    private JButton _btnNewTask      = null;
    private JButton _btnEditTask     = null;
    private JButton _btnCutTask      = null;
    private JButton _btnPasteTask    = null;
    private JButton _btnNewProject   = null;
    private JButton _btnEditProject  = null;
    private JButton _btnCutProject   = null;
    private JButton _btnPasteProject = null;
    private JButton _btnDependencies = null;
    private ToDoTask _pToDoTask = null;
    private CardLayout _pCardLayout = null;
    private JPanel _pnlToDoListOrTask = null;
    private JTextField _txtTask = null;
    private JTextArea _txtComment = null;
    private JTextField _txtStart = null;
    private JTextField _txtWorkStart = null;
    private JTextField _txtDeadline = null;
    private JTextField _txtFinished = null;
    private JTextField _txtClosed = null;
    private JComboBox _cmbPriority = null;
    private OKCancelPanel _ocpTask = null;
    private JList _lstToDo = null;
    private JTextArea _txtNotes = null;
    private ViewController _pViewController = null;
    private JSplitPane _pSplitPane = null;

    /**
     * We want to rely on the fact that always one item is
     * selected in the tree. Also, always at least one node
     * has to be present in the tree.
     */
    private JTree _trToDos                 = null;
    private DefaultMutableTreeNode _tnRoot = null;
    private DefaultMutableTreeNode _ndLast = null;
    private JTextField _txtInput           = null;
    private OKCancelPanel _pOKCancelPanel  = null;

    private static final JDialog DLG_DUMMY = new JDialog();
    private WindowAdapter _waOKCancelPanel = null;
    private WindowAdapter _waOKCancelTask  = null;
    private boolean _bTaskVisible          = false;
    private boolean _bInputVisible         = false;
    private boolean _bInputVisibleBefore   = false;

    private boolean _bEditProjectName      = false;

    private int                    _state        = STATE_STARTUP;
    private DefaultMutableTreeNode _tnCutProject = null;
    private ToDoTask               _cutToDoTask  = null;

    /**
     * This method en- and disables buttons etc. depending
     * on the current state of the application.<p>
     *
     * At appropriate places in the code this method has
     * to be invoked to keep the state uptodate.
     */
    private void _setState( int state_ ) {
        if ( state_ == STATE_NORMAL ) {
            // Ensable all buttons except ok and cancel.
            // Ensable tree as well.

            boolean bTaskSelected = _lstToDo
                                    .getSelectedIndex() 
                                     != -1;

            // one project should always be selected in tree
            _btnNewProject  .setEnabled( true                  );
            _btnEditProject .setEnabled( true                  );
            _btnCutProject  .setEnabled( true                  );
            _btnPasteProject.setEnabled( _tnCutProject != null );
            _btnNewTask     .setEnabled( true                  );
            _btnEditTask    .setEnabled( bTaskSelected         );
            _btnCutTask     .setEnabled( bTaskSelected         );
            _btnPasteTask   .setEnabled( _cutToDoTask != null  );

            _trToDos        .setEnabled( true                  );
        } else if ( state_ == STATE_INPUT ) {
            Util.panicIf( _state != STATE_NORMAL );

            // Disable all buttons except ok and cancel.
            // Disable tree as well.
            _btnNewProject  .setEnabled( false );
            _btnEditProject .setEnabled( false );
            _btnCutProject  .setEnabled( false );
            _btnPasteProject.setEnabled( false );
            _btnNewTask     .setEnabled( false );
            _btnEditTask    .setEnabled( false );
            _btnCutTask     .setEnabled( false );
            _btnPasteTask   .setEnabled( false );

            _trToDos        .setEnabled( false );
        } 

        _state = state_;
    }

    private void _setInputVisible( boolean bVisible_ ) {
        if ( _bInputVisible == bVisible_ ) {
            // nothing to do
            return;
        }

        String sProjectName = _txtInput.getText();
        if ( _bEditProjectName ) {
            sProjectName = _getSelectedProject().toString();
            Util.debug( this
                        , "_setInputVisible(..).sProjectName: "
                          + sProjectName );
        }

        _txtInput.setText   ( sProjectName );
        _txtInput.setVisible( bVisible_    );
        _pOKCancelPanel.setVisible( bVisible_ );
        _txtInput.invalidate();
        _pOKCancelPanel.invalidate();
        validate();
        _pOKCancelPanel.validate();

        if ( bVisible_ ) {
            _setState( STATE_INPUT );
            _txtInput.requestFocus();
            _waOKCancelPanel.windowOpened( null );
            _bInputVisibleBefore = true;
        } else {
            _setState( STATE_NORMAL );
            if ( _bInputVisibleBefore ) {
                _waOKCancelPanel.windowClosed( null );
                _bInputVisibleBefore = false;
            }
        }

        if ( _trToDos != null && _txtInput != null ) {
            _trToDos.setEnabled( !bVisible_ );
            _txtNotes.setEnabled( !bVisible_ );
        }

        _bInputVisible = bVisible_;
    }

    private void _sort() {
	Util.debug( this, "_sort().START: " + (new Date()) );
        // create comparator
        Comparable pComparable = new ToDoComparable();
        /*Comparable() {
            public int compare( Object oFirst_, Object oSecond_ ) {
                ToDoTask tdFirst = (ToDoTask)oFirst_;
                ToDoTask tdSecond = (ToDoTask)oSecond_;
		//Util.debug( this, "compare(..).tdFirst: " +
		//	    tdFirst );
		//Util.debug( this, "compare(..).tdSecond: " +
		//tdSecond );
                int retVal = tdFirst.getFinished().compareTo( tdSecond.getFinished() );
                if ( retVal == 0 ) {
                    String sDeadline1 = tdFirst.getDeadline();
                    String sDeadline2 = tdSecond.getDeadline();
                    if ( Util.isEmpty( sDeadline1 ) ) {
                        sDeadline1 = "Z";
                    }
                    if ( Util.isEmpty( sDeadline2 ) ) {
                        sDeadline2 = "Z";
                    }
                    retVal = sDeadline1.compareTo( sDeadline2 );
                }
                if ( retVal == 0 ) {
                    retVal = tdFirst.getPriority().compareTo( tdSecond.getPriority() );
                }
                if ( retVal == 0 ) {
                    retVal = tdFirst.getStart().compareTo( tdSecond.getStart() );
                }
                if ( retVal == 0 ) {
                    retVal = tdFirst.getTitle().compareTo( tdSecond.getTitle() );
                }
		//Util.debug( this, "compare(..).retVal: " +
		//  retVal );

                return retVal;
            }
        };*/

	// get selected node
	DefaultMutableTreeNode pSelectedNode = (DefaultMutableTreeNode)_trToDos.
	    getLastSelectedPathComponent();
        if ( pSelectedNode == null ) {
	    Util.debug( this, "_sort().NOTHING_SELECFTED" );

            return;
        }
        Vector vToDoTasks = (Vector)pSelectedNode.getUserObject();
	Util.debug( this, "_sort().vToDoTasks.size: " + 
		    vToDoTasks.size() );

	// sort
	Util.sortFast( vToDoTasks, pComparable );
	pSelectedNode.setUserObject( vToDoTasks );

	// fill list model
        DefaultListModel dlmToDo = (DefaultListModel)_lstToDo.getModel();
        /*Enumeration eToDoTasks = dlmToDo.elements();
        Vector vToDoTasks = Util.sort( eToDoTasks, pComparable );*/
        _lstToDo.clearSelection();
        dlmToDo.removeAllElements();
        Enumeration eToDoTasks = vToDoTasks.elements();
	int index = 0;
        while( eToDoTasks.hasMoreElements() ) {
	    index++;
            dlmToDo.addElement( "" + index + " " + eToDoTasks.nextElement().toString() );
        }
	Util.debug( this, "_sort().END: " + (new Date()) );
    }

        // create comparator
    public static class ToDoComparable implements Comparable {
	public int compare( Object oFirst_, Object oSecond_ ) {
	    ToDoTask tdFirst = (ToDoTask)oFirst_;
	    ToDoTask tdSecond = (ToDoTask)oSecond_;
	    /*Util.debug( this, "compare(..).tdFirst: " +
	      tdFirst );
	      Util.debug( this, "compare(..).tdSecond: " +
	      tdSecond );*/
	    int retVal = tdFirst.getEnd().compareTo( tdSecond.getEnd() );
            // for closed and finished, order is reverse
            if ( (!tdFirst.isOpen()) && (!tdSecond.isOpen()) ) {
                retVal *= -1;
            }
	    if ( retVal == 0 ) {
		String sDeadline1 = tdFirst.getDeadline();
		String sDeadline2 = tdSecond.getDeadline();
		if ( Util.isEmpty( sDeadline1 ) ) {
		    sDeadline1 = "z";
		}
		if ( Util.isEmpty( sDeadline2 ) ) {
		    sDeadline2 = "z";
		}
		retVal = sDeadline1.compareTo( sDeadline2 );
	    }
	    if ( retVal == 0 ) {
		retVal = tdFirst.getPriority().compareTo( tdSecond.getPriority() );
	    }
	    if ( retVal == 0 ) {
		retVal = tdFirst.getStart().compareTo( tdSecond.getStart() );
	    }
	    if ( retVal == 0 ) {
		retVal = tdFirst.getTitle().compareTo( tdSecond.getTitle() );
	    }
	    /*Util.debug( this, "compare(..).retVal: " +
	      retVal );*/
	    
	    return retVal;
	}
    };

    public static Comparable _pComparable = new ToDoComparable();

    private void _insert( ToDoTask pToDoTask_ ) {
	Util.debug( this, "_insert().START: " + (new Date()) );

	// get selected node
	DefaultMutableTreeNode pSelectedNode = (DefaultMutableTreeNode)_trToDos.
	    getLastSelectedPathComponent();
        if ( pSelectedNode == null ) {
	    Util.debug( this, "_insert().NOTHING_SELECFTED" );

            return;
        }
        Vector vToDoTasks = (Vector)pSelectedNode.getUserObject();
	Util.debug( this, "_insert().vToDoTasks.size: " + 
		    vToDoTasks.size() );

	// sort
	int selectedIndex = Util.insert( vToDoTasks, pToDoTask_, _pComparable );
	pSelectedNode.setUserObject( vToDoTasks );

	// fill list model
        DefaultListModel dlmToDo = (DefaultListModel)_lstToDo.getModel();
        /*Enumeration eToDoTasks = dlmToDo.elements();
	  Vector vToDoTasks = Util.sort( eToDoTasks, pComparable );*/
        /*_lstToDo.clearSelection();
        dlmToDo.removeAllElements();
        Enumeration eToDoTasks = vToDoTasks.elements();
	int index = 0;
        while( eToDoTasks.hasMoreElements() ) {
	    index++;
            dlmToDo.addElement( "" + index + " " + eToDoTasks.nextElement().toString() );
	    }*/
	dlmToDo.insertElementAt( "" + (selectedIndex + 1) + " " + pToDoTask_.toString(), selectedIndex );
	for( int index = 0; index < vToDoTasks.size(); index++ ) {
	    dlmToDo.setElementAt( "" + (index + 1) + " " + vToDoTasks.elementAt( index ), index );
	}
	_lstToDo.setSelectedIndex( selectedIndex );
	Util.debug( this, "_insert().END: " + (new Date()) );
    }


    private void _setTaskVisible( boolean bVisible_ ) {
        if ( _bTaskVisible == bVisible_ ) {
            // nothing to do
            return;
        }

        if ( bVisible_ ) {
            _setState( STATE_TASK );
            _trToDos.setEnabled( false );
            _pCardLayout.show( _pnlToDoListOrTask, S_TASK_NAME );
            _txtStart.setText( _pToDoTask.getStart() );

            _lblId.setText( Util.ltoa( _pToDoTask.getId() ) );

            _txtDependsOn.setText( _pToDoTask.getDependenciesString() );
            _txtBlocks   .setText( _pToDoTask.getBlocksString      () );

            _txtTask.setText( _pToDoTask.getTitle() );
            _txtTask.requestFocus();

            _txtComment.setText( _pToDoTask.getComment() );
            _txtClosed.setText( _pToDoTask.getClosed() );
            Util.debug( this, "_setTaskVisible(..)._pToDoTask.getFinished(): --->" +
                        _pToDoTask.getFinished() + "<---" );
            _txtFinished.setText( _pToDoTask.getFinished() );
            _txtDeadline.setText( _pToDoTask.getDeadline() );
            _cmbPriority.setSelectedItem( _pToDoTask.getPriority() );
            //_btnNewTask .setEnabled( false );
            //_btnEditTask.setEnabled( false );
            //_btnCutTask .setEnabled( false );
        } else {
            _setState( STATE_NORMAL );
            _pCardLayout.show( _pnlToDoListOrTask, S_LIST_NAME );
            //_btnNewTask .setEnabled( true );
            //_btnEditTask.setEnabled( true );
	    //_btnCutTask .setEnabled( true );
            //_trToDos.setEnabled( true );
        }

        _bTaskVisible = bVisible_;
    }

    /**
     * Use this method to also get the project name,
     * just append a 'toString()' method.<p>
     *
     * You receive the original object, not a cloned object.
     */
    private ToDoTaskVector _getSelectedProject() {
	DefaultMutableTreeNode pSelectedNode = (DefaultMutableTreeNode)_trToDos.
	    getLastSelectedPathComponent();
        if ( pSelectedNode == null ) {
            return null;
        }

        ToDoTaskVector vToDoTasks = (ToDoTaskVector)pSelectedNode.getUserObject();
	
	return vToDoTasks;
    }

    private void _commitTask() {
	Util.debug( this, "_commitTask().START: " + (new Date()) );
	_pViewController.setModified( true );
        _pToDoTask.setStart( _txtStart.getText() );
        _pToDoTask.setTitle( _txtTask.getText() );
        _pToDoTask.setComment( _txtComment.getText() );
        _pToDoTask.setClosed( _txtClosed.getText() );
        _pToDoTask.setFinished( _txtFinished.getText() );
        _pToDoTask.setDeadline( _txtDeadline.getText() );
        _pToDoTask.setPriority( (ToDoTask.Priority)_cmbPriority.getSelectedItem() );

        _pToDoTask.setDependencies( _txtDependsOn.getText() );
        _pToDoTask.setBlocks      ( _txtBlocks   .getText() );

        // is it a new task?
	ToDoTaskVector vToDoTaskVector = _getSelectedProject();
	Util.panicIf ( vToDoTaskVector == null );
	int index = vToDoTaskVector.indexOf( _pToDoTask );
        if ( index != -1 ) {
            vToDoTaskVector.removeElementAt( index );
	    DefaultListModel dlmToDo = (DefaultListModel)_lstToDo.getModel();
	    dlmToDo.removeElementAt( index );
        }

	_insert( _pToDoTask );
        /*_sort();
	int selectIndex = _getSelectedProject().indexOf( _pToDoTask );
        _lstToDo.setSelectedIndex( selectIndex );*/
	Util.debug( this, "_commitTask().END: " + (new Date()) );
    }

    private JTextField _txtDependsOn = null;
    private JTextField _txtBlocks    = null;
    private JLabel     _lblId        = null;

    private JPanel _createToDoTaskPane() {
        String sTimezonID = _pViewController.getInit().
               getKeyValue( "TimezoneID" ); 

        _txtStart = new DateField( sTimezonID );
        _txtStart.setName         ( "StartText" );
        _txtStart.setMinimumSize  ( new Dimension( 100, 24 ) );
        _txtStart.setPreferredSize( new Dimension( 100, 24 ) );

        _lblId = new JLabel( "XXXXXX" );

        _txtWorkStart = new DateField( sTimezonID );
        _txtWorkStart.setMinimumSize  ( new Dimension( 100, 24 ) );
        _txtWorkStart.setPreferredSize( new Dimension( 100, 24 ) );

        _txtDeadline = new DateField( sTimezonID );
        _txtDeadline.setMinimumSize  ( new Dimension( 100, 24 ) );
        _txtDeadline.setPreferredSize( new Dimension( 100, 24 ) );

        _txtTask = new JTextField();
        _txtTask.setName         ( "TitleText"                 );
        _txtTask.setBorder       ( SwingUtil.createCCLBorder() );
        _txtTask.setPreferredSize( new Dimension( 100, 24 )    );

        _txtComment = new JTextArea();
        _txtComment.setName( "CommentText" );
        _txtComment.setBackground( Color.white );
        JScrollPane scpComment = new JScrollPane( _txtComment );
        scpComment.setBorder( SwingUtil.createCCLBorder() );
        _pViewController.addDateAction( _txtComment );

        _cmbPriority = new JComboBox();
        _cmbPriority.addItem( ToDoTask.A );
        _cmbPriority.addItem( ToDoTask.B );
        _cmbPriority.addItem( ToDoTask.C );
        _cmbPriority.addItem( ToDoTask.D );

        _txtClosed = new DateField( sTimezonID );
        _txtClosed.setMinimumSize  ( new Dimension( 100, 24 ) );
        _txtClosed.setPreferredSize( new Dimension( 100, 24 ) );

        _txtFinished = new DateField( sTimezonID );
        _txtFinished.setName         ( "FinishedText"           );
        _txtFinished.setMinimumSize  ( new Dimension( 100, 24 ) );
        _txtFinished.setPreferredSize( new Dimension( 100, 24 ) );

        _txtDependsOn = new JTextField();
        _txtBlocks    = new JTextField();
        _txtDependsOn.setBorder( SwingUtil.createCCLBorder() );
        _txtBlocks   .setBorder( SwingUtil.createCCLBorder() );

        _ocpTask = new OKCancelPanel( DLG_DUMMY, 0 );
        JButton btnOK = _ocpTask.getOKButton();
        _pViewController.getRootPane().setDefaultButton( btnOK );
        _waOKCancelTask = _pOKCancelPanel.getWindowAdapter();
        DLG_DUMMY.removeWindowListener( _waOKCancelTask );

        btnOK.addActionListener
               ( new ActionListener() {
                   public void actionPerformed( ActionEvent pActionEvent_ ) {
                       _commitTask();
                       _setTaskVisible( false );
                   }
               } );
        _ocpTask.getCancelButton().addActionListener
               ( new ActionListener() {
                   public void actionPerformed( ActionEvent pActionEvent_ ) {
                       _setTaskVisible( false );
                   }
               } );


        JPanel pnlToDoTask = new JPanel();
        pnlToDoTask.setName( S_TASK_NAME );
        pnlToDoTask.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );

        AutoGridBagLayout agblTask = new AutoGridBagLayout();

        agblTask.setLayoutOn        ( pnlToDoTask                       );
        agblTask.setAnchor          ( AutoGridBagLayout.WEST            );

        // start date
        agblTask.add                ( pnlToDoTask
                                      , new JLabel( "Start date:   " )  );
        agblTask.add                ( pnlToDoTask
                                      , _txtStart                       );

        // Id
        agblTask.skip               ();

        agblTask.setAnchor          ( AutoGridBagLayout.EAST            );
        agblTask.setExpandHorizontal();
        agblTask.add                ( pnlToDoTask
                                      , new JLabel( "Id:   " )          );
        agblTask.setExpandNone      ();
        agblTask.add                ( pnlToDoTask, _lblId               );
        agblTask.setAnchor          ( AutoGridBagLayout.WEST            );
        agblTask.endLine            ();

        // work start
        agblTask.add                ( pnlToDoTask
                                      , new JLabel( "Work start:   " )  );
        agblTask.add                ( pnlToDoTask, _txtWorkStart        );
        agblTask.endLine            ();

        // deadline
        agblTask.add                ( pnlToDoTask
                                      , new JLabel( "Deadline:   " )    );
        agblTask.setFillVertical    ();
        agblTask.add                ( pnlToDoTask, _txtDeadline         );
        agblTask.setFillNone        ();

        agblTask.endLine            ();

        // title
        agblTask.add                ( pnlToDoTask
                                      , new JLabel( "Task:   " )        );
        agblTask.setFillHorizontal  ();
        agblTask.setExpandHorizontal();
        agblTask.setExtend          ( 4, 1                              );
        agblTask.add                ( pnlToDoTask, _txtTask             );
        agblTask.setExtend          ( 1, 1                              );
        agblTask.setFillNone        ();
        agblTask.setExpandNone      ();
        agblTask.endLine            ();

        // comment
        agblTask.setAnchor          ( AutoGridBagLayout.NORTHWEST       );
        agblTask.add                ( pnlToDoTask
                                      , new JLabel( "Comment:   " )     );
        agblTask.setAnchor          ( AutoGridBagLayout.WEST            );
        agblTask.setFillBoth        ();
        agblTask.setExpandBoth      ();
        agblTask.setExtend          ( 4, 1                              );
        agblTask.add                ( pnlToDoTask, scpComment           );
        agblTask.setExtend          ( 1, 1                              );
        agblTask.setFillNone        ();
        agblTask.setExpandNone      ();
        agblTask.endLine            ();

        // priority
        agblTask.add                ( pnlToDoTask
                                      , new JLabel( "Priority:   " )    );
        agblTask.add                ( pnlToDoTask, _cmbPriority         );
        agblTask.endLine();

        // closed
        agblTask.add                ( pnlToDoTask
                                      , new JLabel( "Closed:   " )      );
        agblTask.setFillVertical    ();
        agblTask.add                ( pnlToDoTask, _txtClosed           );
        agblTask.setFillNone        ();

        // Depends on
        agblTask.add                ( pnlToDoTask
                                      , new JLabel( "  Depends on: " )  );
        agblTask.setFillBoth        ();
        agblTask.setExpandHorizontal();
        agblTask.setExtend          ( 2, 1                              );
        agblTask.add                ( pnlToDoTask, _txtDependsOn        );
        agblTask.setExtend          ( 1, 1                              );
        agblTask.setExpandNone      ();
        agblTask.setFillNone        ();
        agblTask.endLine            ();

        // finished
        agblTask.add                ( pnlToDoTask
                                      , new JLabel( "Finished:   " )    );
        agblTask.setFillVertical    ();
        agblTask.add                ( pnlToDoTask, _txtFinished         );
        agblTask.setFillNone        ();

        // Blocks
        agblTask.add                ( pnlToDoTask
                                      , new JLabel( "  Blocks: " )      );
        agblTask.setFillBoth        ();
        agblTask.setExpandHorizontal();
        agblTask.setExtend          ( 2, 1                              );
        agblTask.add                ( pnlToDoTask, _txtBlocks           );
        agblTask.setExtend          ( 1, 1                              );
        agblTask.setExpandNone      ();
        agblTask.setFillNone        ();
        agblTask.endLine            ();


        // OK or cancel
        agblTask.add                ( pnlToDoTask, new JLabel( " " )    );
        agblTask.endLine            ();
        
        agblTask.setAnchor          ( GridBagConstraints.EAST           );
        
        agblTask.setExtend          ( 5, 1                              );
        pnlToDoTask.add             ( _ocpTask                          );
        agblTask.setExtend          ( 1, 1                              );
        agblTask.endLine            ();
        
        return pnlToDoTask;
    }

    private String _getTimezoneID() {
        return _pViewController.getInit().getKeyValue( "TimezoneID" );
    }

    public ToDoPanel( ViewController pViewController_ ) {
        super();

        _pViewController = pViewController_;

        AutoGridBagLayout pAutoGridBagLayout = 
               new AutoGridBagLayout( 2 );

        setLayout( pAutoGridBagLayout );
        setBorder( BorderFactory.createEmptyBorder( 7, 5, 5, 5 ) );
        pAutoGridBagLayout.setAnchor( GridBagConstraints.WEST );

        // buttons
        {
            _btnNewProject = new JButton( "New Project" );
            _btnNewProject.setName( "AddButton" );
            _btnNewProject.addActionListener
                   ( new ActionListener() {
                       public void actionPerformed( ActionEvent pActionEvent_ ) {
                           _setInputVisible( true );
                       }
                   } );
            add( _btnNewProject );
            
            _btnEditProject = new JButton( "Edit Project" );
            _btnEditProject.setName( "EditButton" );
            _btnEditProject.addActionListener
                   ( new ActionListener() {
                       public void actionPerformed( ActionEvent pActionEvent_ ) {
                           _bEditProjectName = true;
                           _setInputVisible( true );
                       }
                   } );
            add( _btnEditProject );
            
            _btnCutProject = new JButton( "Cut Project" );
            _btnCutProject.setName( "DeleteButton" );
            _btnCutProject.addActionListener
                   ( new ActionListener() {
                       public void actionPerformed( ActionEvent pActionEvent_ ) {
                           _setInputVisible( false );
                           deleteNote();
                       }
                   } );
            pAutoGridBagLayout.setFillHorizontal();
            add( _btnCutProject );
            
            _btnPasteProject = new JButton( "Paste Project" );
            _btnPasteProject.setName( "PasteButton" );
            _btnPasteProject.setEnabled( false );
            _btnPasteProject.addActionListener
                   ( new ActionListener() {
                       public void actionPerformed( ActionEvent pActionEvent_ ) {
                           _setInputVisible( false );
                           pasteProject();
                       }
                   } );
            add( _btnPasteProject );
        }

        // input field
        {
            add( new JLabel( " " ) );

            _txtInput = new JTextField();
            _txtInput.setName( "InputText" );
            _txtInput.setBorder( SwingUtil.createCCLBorder() );
            pAutoGridBagLayout.setExpandHorizontal();
            pAutoGridBagLayout.setFillHorizontal();
            add( _txtInput );
            pAutoGridBagLayout.setExpandNone();
            pAutoGridBagLayout.setFillNone();
            
            _pOKCancelPanel = new OKCancelPanel( DLG_DUMMY, 0 );
            JButton btnOK = _pOKCancelPanel.getOKButton();
            _pViewController.getRootPane().setDefaultButton( btnOK );
            _waOKCancelPanel = _pOKCancelPanel.getWindowAdapter();
            DLG_DUMMY.removeWindowListener( _waOKCancelPanel );
            add( _pOKCancelPanel );

            _pOKCancelPanel.getOKButton().addActionListener
                   ( new ActionListener() {
                       public void actionPerformed( ActionEvent pActionEvent_ ) {
                           String sInput = _txtInput.getText().trim();
                           if ( _bEditProjectName ) {
                               editProject( sInput );
                               _bEditProjectName = false;
                           } else {
                               createNewProject( sInput );
                           }
                           _setInputVisible( false );
                       }
                   } );
            _pOKCancelPanel.getCancelButton().addActionListener
                   ( new ActionListener() {
                       public void actionPerformed( ActionEvent pActionEvent_ ) {
                           _setInputVisible( false );
                       }
                   } );
            pAutoGridBagLayout.endLine();
        }

        // ToDoListPanel
        JScrollPane scpToDoList = null;
        {
            DefaultListModel dlmToDo = new DefaultListModel();
            _lstToDo = new JList( dlmToDo );
            _lstToDo.setName( "ToDoList" );
            _lstToDo.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
            _lstToDo.addListSelectionListener( this );
            scpToDoList = new JScrollPane( _lstToDo );
            scpToDoList.setBorder( SwingUtil.createCCLBorder() );
        }

        // ToDoTaskPanel
        JPanel pnlToDoTask = _createToDoTaskPane();

        // ToDoListOrTaskPanel
        {
            _pnlToDoListOrTask = new JPanel();
            _pCardLayout = new CardLayout();
            _pnlToDoListOrTask.setLayout( _pCardLayout );
            _pnlToDoListOrTask.add( scpToDoList, S_LIST_NAME );
            _pnlToDoListOrTask.add( pnlToDoTask, S_TASK_NAME );
        }

        // todo buttons
        {
            pAutoGridBagLayout.setFillHorizontal();
            _btnNewTask = new JButton( "New Task" );
            _btnNewTask.setName( "NewTask" );
            _btnNewTask.addActionListener
                   ( new ActionListener() {
                       public void actionPerformed( ActionEvent pActionEvent_ ) {
                           _pToDoTask = new ToDoTask( _getTimezoneID() );
                           _setTaskVisible( true );
                       }
                   } );
            add( _btnNewTask );

            _btnEditTask = new JButton( "Edit Task" );
            _btnEditTask.setName( "EditTask" );
            _btnEditTask.addActionListener
                   ( new ActionListener() {
                       public void actionPerformed( ActionEvent pActionEvent_ ) {
			   ToDoTaskVector vToDoTaskVector = _getSelectedProject();
			   int index = _lstToDo.getSelectedIndex();
                           if ( index == -1 ) {
                               SwingUtil.showMessage( _pViewController,
                                                      "No ToDo-task selected!" );

                               return;
                           }
                           _pToDoTask = (ToDoTask)vToDoTaskVector.elementAt( index );
                           _setTaskVisible( true );
                       }
                   } );
            add( _btnEditTask );

            _btnCutTask = new JButton( "Cut Task" );
            _btnCutTask.setName( "DeleteTask" );
            _btnCutTask.addActionListener
                   ( new ActionListener() {
                       public void actionPerformed( ActionEvent pActionEvent_ ) {
                           cutTask();
                       }
                   } );
            add( _btnCutTask );

            _btnPasteTask = new JButton( "Paste Task" );
            _btnPasteTask.setName( "PasteTask" );
            _btnPasteTask.addActionListener
                   ( new ActionListener() {
                       public void actionPerformed( ActionEvent pActionEvent_ ) {
                           pasteTask();
                       }
                   } );
            add( _btnPasteTask );

            pAutoGridBagLayout.endLine();
        }

        _txtNotes = new JTextArea();
        JScrollPane scpText = new JScrollPane( _txtNotes );
        scpText.setBorder( SwingUtil.createCCLBorder() );

        DocumentListener pDocumentListener = new DocumentListener() {
            public void insertUpdate(DocumentEvent e) {
                if ( _bValueChanged ) {
                    return;
                }
                _pViewController.stateChanged( null );
            }

            public void removeUpdate(DocumentEvent e) {
                if ( _bValueChanged ) {
                    return;
                }
                _pViewController.stateChanged( null );
            }

            public void changedUpdate(DocumentEvent e) {
                if ( _bValueChanged ) {
                    return;
                }
                _pViewController.stateChanged( null );
            }
        };
        _txtNotes.getDocument().addDocumentListener( pDocumentListener );
        _txtNotes.setName( "_txtNotes" );

        // set date insertion text action
        TextAction pDateTextAction = new DateTextAction();
        Keymap kmOld = _txtNotes.getKeymap();
        Keymap kmNew = _txtNotes.addKeymap( null, kmOld );
        KeyStroke ksCtrlD = KeyStroke.getKeyStroke
               ( KeyEvent.VK_D, Event.CTRL_MASK );
        kmNew.removeKeyStrokeBinding( ksCtrlD );
        kmNew.addActionForKeyStroke
               ( ksCtrlD, pDateTextAction);
        _txtNotes.setKeymap( kmNew );

        _tnRoot = createRootNode();
        _trToDos = new JTree( _tnRoot );
        _trToDos.setName( "ToDoTree" );
        _trToDos.getSelectionModel().
               setSelectionMode( TreeSelectionModel.SINGLE_TREE_SELECTION );
        JScrollPane scpTree = new JScrollPane( _trToDos );
        scpTree.setBorder( SwingUtil.createCCLBorder() );

        _trToDos.addTreeSelectionListener( this );
        _trToDos.setSelectionInterval( 0, 0 );

        _pSplitPane = new JSplitPane();
        _pSplitPane.setBorder( BorderFactory.createEmptyBorder( 3, 0, 0, 0 ) );
        _pSplitPane.setLeftComponent( scpTree );
        _pSplitPane.setRightComponent( _pnlToDoListOrTask );
        _pSplitPane.setDividerSize( 2 );
        _pSplitPane.setDividerLocation( 300 );
        Util.debug( this, "<init>(..).splitpane.size: " +
                    _pSplitPane.getSize() );
        Util.debug( this, "<init>(..).splitpane.dividerLocation: " +
                    _pSplitPane.getDividerLocation() );

        pAutoGridBagLayout.setExtend( 7, 1 );
        pAutoGridBagLayout.setExpandBoth();
        pAutoGridBagLayout.setFillBoth();

        add( _pSplitPane );

        // true necessary or set visible will have no effect
        _bInputVisible = true;
        _setInputVisible( false );
    }

    public boolean isTaskVisible() {
        return _bTaskVisible;
    }

    public void deleteNote() {
        // might not be necessary, but can't hurt
        saveLastNode();

        // get selected note
        TreePath pTreePath = _trToDos.getSelectionPath();
        DefaultMutableTreeNode tnSelected = 
               (DefaultMutableTreeNode)
               pTreePath.getLastPathComponent();

        // don't delete top node
        DefaultMutableTreeNode tnParent = (DefaultMutableTreeNode)tnSelected.getParent();
        if ( tnParent == null ) {
            SwingUtil.showMessage( _pViewController, "Sorry, you can't delete this project!" );
            
            return;
        }

        // sure to delete this note?
        boolean bDelete = SwingUtil.isOKOrCancel
               ( _pViewController, 
                 "Is it OK to delete note\n'" +
                 tnSelected.toString() +
                 "'\nand all of its sub projects?" );

        _tnCutProject = tnSelected;

        // delete it
        if ( bDelete ) {
            DefaultTreeModel pModel = (DefaultTreeModel)_trToDos.getModel();
            TreePath tpParent = pTreePath.getParentPath();
            pModel.removeNodeFromParent( tnSelected );
            
            // make sure something is selected
            _trToDos.setSelectionPath( tpParent );
        }

        _pViewController.setModified( true );
        _btnPasteProject.setEnabled( true );
    }

    public void pasteProject() {
        saveLastNode();

        if ( _tnCutProject == null ) {
            // nothing to paste
            return;
        }

        // find parent
        TreePath pTreePath = _trToDos.getSelectionPath();
        DefaultMutableTreeNode tnParent = (DefaultMutableTreeNode)
               pTreePath.getLastPathComponent();

        // find new index
        Vector pVector = (Vector)_tnCutProject.getUserObject();
        int newIndex = 0;
        Enumeration eChildren = tnParent.children();
        while( eChildren.hasMoreElements() ) {
            DefaultMutableTreeNode tnNext = (DefaultMutableTreeNode)
                   eChildren.nextElement();
            if ( pVector.toString().compareTo
                 ( tnNext.getUserObject().toString() ) < 0 )
            {
                break;
            }
            newIndex++;
        }

        // insert
        DefaultMutableTreeNode tnNew =
               cloneProjectNode( _tnCutProject );
        ((DefaultTreeModel)_trToDos.getModel()).
               insertNodeInto( tnNew, tnParent, newIndex );

        _pViewController.setModified( true );
    }

    public void createNewProject( String sKey_ ) {
        ToDoTaskVector vProject = new ToDoTaskVector( sKey_ );
        
        // find parent
        TreePath pTreePath = _trToDos.getSelectionPath();
        DefaultMutableTreeNode tnParent = (DefaultMutableTreeNode)
               pTreePath.getLastPathComponent();

        // find new index
        int newIndex = 0;
        Enumeration eChildren = tnParent.children();
        while( eChildren.hasMoreElements() ) {
            DefaultMutableTreeNode tnNext = (DefaultMutableTreeNode)
                   eChildren.nextElement();
            if ( vProject.toString().compareTo
                 ( tnNext.getUserObject().toString() ) < 0 )
            {
                break;
            }
            newIndex++;
        }

        // insert
        DefaultMutableTreeNode tnNew =
               new DefaultMutableTreeNode( vProject );
        ((DefaultTreeModel)_trToDos.getModel()).
               insertNodeInto( tnNew, tnParent, newIndex );

        _pViewController.setModified( true );
    }

    public void editProject( String sKey_ ) {
        ToDoTaskVector vProject = _getSelectedProject();
        
        vProject.setName( sKey_ );

        // update tree view
        TreePath pTreePath = _trToDos.getSelectionPath();
        DefaultMutableTreeNode tnSelected = (DefaultMutableTreeNode)
               pTreePath.getLastPathComponent();
        ((DefaultTreeModel)_trToDos.getModel())
               .valueForPathChanged( pTreePath
                                     , vProject );
        
        _pViewController.setModified( true );
    }

    public synchronized void saveLastNode() {
	/*
	  // no real need to save this,
	  // because we always work on vector directly and use list just for presentation.
	  if ( _ndLast != null ) {
	  Vector vProject = (Vector)_ndLast.getUserObject();
	  vProject.removeAllElements();
	  DefaultListModel dlmToDo = (DefaultListModel)_lstToDo.getModel();
	  Enumeration pEnumeration = dlmToDo.elements();
	  while( pEnumeration.hasMoreElements() ) {
	  vProject.addElement( pEnumeration.nextElement() );
	  }
	  }*/
    }

    public void cutTask() {
        _pViewController.setModified( true );
        ToDoTaskVector pToDoTaskVector = _getSelectedProject();
        int index = _lstToDo.getSelectedIndex();
        if ( index == -1 ) {
            Util.printlnErr( "Cut ToDo Task should not be enabled if no task is selected!" );

            SwingUtil.showMessage( _pViewController,
                                   "No ToDo-task selected to cut!" );
            
            return;
        }

        _cutToDoTask = (ToDoTask)((ToDoTask)pToDoTaskVector.elementAt( index )).clone();
        pToDoTaskVector.removeElementAt( index );

        DefaultListModel dlmToDo = (DefaultListModel)_lstToDo.getModel();
        dlmToDo.removeElementAt( index );
        while( index < pToDoTaskVector.size() ) {
            dlmToDo.setElementAt( "" + (index + 1) + " " + pToDoTaskVector.elementAt( index ), index );
            index++;
        }

        _setState( STATE_NORMAL );
    }

    public void pasteTask() {
        if ( _cutToDoTask == null ) {
            // nothing to paste
            Util.printlnErr( "Paste ToDo Task button should be disabled if nothing is to be pasted!" );

            return;
        }

        _pViewController.setModified( true );

        // insert task into project
        _insert( (ToDoTask)_cutToDoTask.clone() );

        _setState( STATE_NORMAL );
    }

    public void insertDate() {
        if ( !_txtNotes.isEnabled() ) {
            return;
        }
        SwingUtil.invokeLaterIfNecessary( new Runnable() {
            public void run() {
                Date pDate = new Date();
                SimpleDateFormat pDateFormat
                       = new SimpleDateFormat ("yyyy-MM-dd");
                _txtNotes.replaceSelection( pDateFormat.format( pDate ) );
                }
        } );
    }

    public class DateTextAction extends TextAction {
        public DateTextAction() {
            super( "DateTextAction" );
        }

        public void actionPerformed( ActionEvent pActionEvent_ ) {
            JTextComponent txtTarget = getTextComponent( pActionEvent_ );
            if ( txtTarget.getName() != null &&
                 txtTarget.getName().equals( "_txtNotes" ) ) 
            {
                insertDate();
            }
        }
    }

    private boolean _bValueChanged = false;

    public synchronized void valueChanged( TreeSelectionEvent pTreeSelectionEvent_ ) {
        //saveLastNode();

        DefaultMutableTreeNode pNode = (DefaultMutableTreeNode)_trToDos.
               getLastSelectedPathComponent();
        if ( pNode == null ) {
            return;
        }
        // set list
        Vector vToDoTasks = (Vector)pNode.getUserObject();
        DefaultListModel dlmToDo = (DefaultListModel)_lstToDo.getModel();
        _lstToDo.clearSelection();
        dlmToDo.removeAllElements();
        Enumeration pEnumeration = vToDoTasks.elements();
	int index = 0;
        while( pEnumeration.hasMoreElements() ) {
	    index++;
            dlmToDo.addElement( "" + index + " " + pEnumeration.nextElement().toString() );
        }
        //_sort();

        _setState( STATE_NORMAL );

        //_ndLast = pNode;
    }

    public void valueChanged( ListSelectionEvent e ) {
        _setState( STATE_NORMAL );
    }

    public static final String getNotesName() {
        return getToDoName();
    }

    public static final String getToDoName() {
        return "ToDo";
    }

    public String getName() {
        return getToDoName();
    }

    public DefaultMutableTreeNode getToDos() {
        saveLastNode();
        
        return _tnRoot;
    }

    public void setNotes( DefaultMutableTreeNode tnRoot_ ) {
        _ndLast = null;
        _tnRoot = tnRoot_;
        _trToDos.setModel( new DefaultTreeModel( _tnRoot ) );
        _trToDos.setSelectionInterval( 0, 0 );
    }

    public static DefaultMutableTreeNode createRootNode() {
        DefaultMutableTreeNode tnRoot = new DefaultMutableTreeNode( new ToDoTaskVector( getToDoName() ) );

        return tnRoot;
    }

    public static DefaultMutableTreeNode cloneProjectNode( DefaultMutableTreeNode tnProject_ ) {
        DefaultMutableTreeNode tnRetVal = null;

        if ( tnProject_ == null ) {
            return tnRetVal;
        }

        // clone vector
        ToDoTaskVector pToDoTaskVector = (ToDoTaskVector)
               (ToDoTaskVector)tnProject_.getUserObject();
        ToDoTaskVector vDestination = new ToDoTaskVector( pToDoTaskVector.toString() );
        Enumeration pEnumeration = pToDoTaskVector.elements();
        while( pEnumeration.hasMoreElements() ) {            
            vDestination.addElement( ((ToDoTask)pEnumeration.nextElement()).clone() );
        }

        tnRetVal = new DefaultMutableTreeNode( vDestination );
        
        // go through children
        for ( int child = 0; child < tnProject_.getChildCount(); child++ ) {
            DefaultMutableTreeNode tnNextChild = (DefaultMutableTreeNode)tnProject_.getChildAt( child );
            DefaultMutableTreeNode tnNextClone = cloneProjectNode( tnNextChild );
            
            // add child to return value
            tnRetVal.add( tnNextClone );
        }

        return tnRetVal;
    }

    /*public Vector getToDoTasks() {
        DefaultListModel dlmToDo = (DefaultListModel)_lstToDo.getModel();

        Vector vRetVal = new Vector();

        Enumeration eToDoTasks = dlmToDo.elements();
        while( eToDoTasks.hasMoreElements() ) {
            vRetVal.addElement( eToDoTasks.nextElement() );
        }

        return vRetVal;
	}*/

    public void setToDos( DefaultMutableTreeNode tnRoot_ ) {
        _tnRoot = tnRoot_;
        _trToDos.setModel( new DefaultTreeModel( _tnRoot ) );
        _trToDos.setSelectionInterval( 0, 0 );
	//_sort();
    }

    public void setToDoTasks( ToDoTaskVector vToDoTasks_ ) {
        DefaultMutableTreeNode tnRoot = new DefaultMutableTreeNode( vToDoTasks_ );
        _tnRoot = tnRoot;
        _trToDos.setModel( new DefaultTreeModel( _tnRoot ) );
        _trToDos.setSelectionInterval( 0, 0 );

        /*_lstToDo.clearSelection();
        DefaultListModel dlmToDo = (DefaultListModel)_lstToDo.getModel();
        dlmToDo.removeAllElements();

        Enumeration eToDoTasks = vToDoTasks_.elements();
        while( eToDoTasks.hasMoreElements() ) {
            dlmToDo.addElement( eToDoTasks.nextElement() );
        }
        _sort();*/
    }
    
    public ToDoTask getToDoTask() {
        return _pToDoTask;
    }

    public void stateChanged( ChangeEvent pChangeEvent_ ) {
        // must be tabbed pane
        JTabbedPane pTabbedPane = (JTabbedPane)pChangeEvent_.getSource();
        if ( pTabbedPane.getSelectedIndex() == 1 ) {
            _pSplitPane.setDividerLocation( 300 );
            pTabbedPane.removeChangeListener( this );
        }
    }
}
