/*
Copyright (C) 2002 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of JIM - Java Information Manager
(http://www.kclee.com/clemens/java/jim/).

Jim is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jim is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jim; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jim.test;

import ccl.swing.OKCancelFocusManager;
import ccl.swing.OKCancelPanel;
import ccl.swing.SwingUtil;
import ccl.util.FileUtil;
import ccl.util.Testable;
import ccl.util.Test;
import ccl.util.Util;
import java.awt.Component;
import java.awt.Container;
import java.awt.Window;
import javax.accessibility.AccessibleContext;
import javax.accessibility.AccessibleText;
import javax.swing.DefaultFocusManager;
import javax.swing.FocusManager;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.MenuElement;
import javax.swing.SwingUtilities;
import javax.swing.text.Document;
import javax.swing.tree.DefaultMutableTreeNode;

import jim.*;

/**
 * Insert your test code into the '_doIt()' method and
 * test bugs with 'bugIf(boolean expression);' or
 * 'bugIf(boolean expression, "comment about what went wrong");'.
 * Then just run the main routine.
 */
public class MainTest extends Test {
    private void _changeSomething( ViewController pViewController_ ) {
        JTextArea pTextComponent = (JTextArea)SwingUtil.getFirstComponent( pViewController_, "_txtNotes" );
        bugIf( pTextComponent == null );
            
        Document pDocument = pTextComponent.getDocument();
        try {
            pDocument.insertString( 0, "x", null );
            bugIf( false );
        } catch( Exception pException ) {
            bugIf( true );
        }
    }

    private void _open( final ViewController pViewController_, 
                        String sProjectFile_ ) 
    {
        Runnable runTemp = new Runnable() {
            public void run() {
                SwingUtilities.invokeLater
                       (new Runnable() {
                           public void run() {
                               pViewController_.open();
                           }
                       } );
            }
        };
        new Thread( runTemp ).start();
        Util.sleep( 3 );
        Window[] apWindow = pViewController_.getOwnedWindows();

        JDialog dlgFileChooser = (JDialog)apWindow[ apWindow.length - 1 ];
        JTextField txtOpenFile = (JTextField)SwingUtil.getFirstComponent
               ( dlgFileChooser,
                 "javax.swing.plaf.metal.MetalFileChooserUI$1" );
        txtOpenFile.setText( sProjectFile_ );
        Util.sleep( 1 );

        Testable pTestable = new Testable() {
            public boolean test( Object oComponent_ ) {
                if ( oComponent_ instanceof JButton &&
                     ((JButton)oComponent_).getText().equals( "Open" ) )
                {
                    return true;
                }
                    
                return false;
            }
        };
        JButton btnOpen = (JButton)SwingUtil.getComponent
               ( dlgFileChooser,
                 pTestable,
                 0 );
        btnOpen.doClick();
        Util.sleep( 2 );
    }

    private void _selectToDoPanel( ViewController pViewController_ ) {
        JTabbedPane pTabbedPane = (JTabbedPane)SwingUtil.getFirstComponent( pViewController_,
                                                                            "TabbedPane" );
        pTabbedPane.setSelectedIndex( 1 );
        Util.sleep( 1 );
    }

    private void _clickOKButton( JPanel pPanel_ ) {
        OKCancelPanel pnlOKCancel = (OKCancelPanel)SwingUtil.getFirstComponent( pPanel_,
                                                                                "OKCancelPanel" );
        bugIf( pnlOKCancel == null );
        JButton btnOK = pnlOKCancel.getOKButton();
        SwingUtil.doClick( btnOK );
        
        Util.sleep( 2 );
    }

    public MainTest() {
        super();
    }
    
    public MainTest(Test pTest_) {
        super(pTest_);
    }

    /**
     * Real testing starts here.
     */
    protected void _doIt() 
        throws Exception
    {
        /*Util.setDebug( true );*/
        // use cases:
        // new file
        // edit something
        // save
        // specify name
        // new file again

        bugIf( FocusManager.getCurrentManager() instanceof
               OKCancelFocusManager );

        // start jim
        String sIniFile = FileUtil.concatPath
               ( getTestClassDirectory(),
                 "jimtest.ini" );
        FileUtil.delete( sIniFile );
        String[] asArg = { "-inifile",
                           sIniFile };
        
        final ViewController pViewController = new ViewController
               ( asArg, "$Header: /home/clemens/src/java/jim/src/jim/test/RCS/MainTest.java,v 1.6 2002/04/21 13:25:27 clemens Exp clemens $" );
        Runnable runJim = new Runnable() {
            public void run() {
                pViewController.start();
            }
        };
        new Thread( runJim ).start();
        Util.sleep( 3 );

        // test title " *" stuff
        JTextArea pTextComponent = null;
        Document pDocument = null;
        {
            String sTitle = pViewController.getTitle();
            bugIf( !sTitle.equals( "Jim" ) );
            
            pTextComponent = (JTextArea)SwingUtil.getFirstComponent( pViewController, "_txtNotes" );
            bugIf( pTextComponent == null );
            
            pDocument = pTextComponent.getDocument();
            try {
                pDocument.insertString( 0, "x", null );
            } catch( Exception pException ) {
            }
            
            // change something
            sTitle = pViewController.getTitle();
            bugIf( !Util.endsWith( sTitle, " *" ) );
            bugIf( !sTitle.startsWith( "Jim" ) );

            // save
            {
                // new project save as jim/test/jimtest.jiml
                String sProjectFile = FileUtil.concatPath
                       ( getTestClassDirectory(), "jimtest.jiml" );
                FileUtil.delete( sProjectFile );
                Runnable runTemp = new Runnable() {
                    public void run() {
                        pViewController.save();
                    }
                };
                new Thread( runTemp ).start();
                Util.sleep( 3 );
                Window[] apWindow = pViewController.getOwnedWindows();
                bugIf( apWindow.length != 1 );
                bugIf( !(apWindow[ 0 ] instanceof JDialog ) );
                Util.debug( "MainTest._doIt().apWindow.length: " +
                            apWindow.length );

                JDialog dlgFileChooser = (JDialog)apWindow[ 0 ];
                JTextField txtSaveFile = (JTextField)SwingUtil.
                       getFirstComponent( dlgFileChooser,
                                          "javax.swing.plaf.metal.MetalFileChooserUI$1" );
                Util.debug( "MainTest._doIt().txtSaveFile: " +
                            txtSaveFile );
                bugIf( txtSaveFile == null );
                Util.sleep( 60 );
                txtSaveFile.setText( sProjectFile );
                Util.sleep( 1 );
                
                Testable pTestable = new Testable() {
                    public boolean test( Object oComponent_ ) {
                        if ( oComponent_ instanceof JButton &&
                             ((JButton)oComponent_).getText().equals( "Save As" ) )
                        {
                            return true;
                        }
                        
                        return false;
                    }
                };
                JButton btnSave = (JButton)SwingUtil.getComponent
                       ( dlgFileChooser,
                         pTestable,
                         0 );
                bugIf( btnSave == null, "Save button is null!" );
                btnSave.doClick();
                Util.sleep( 2 );

                sTitle = pViewController.getTitle();
                bugIf( !sTitle.equals( "Jim: Jimtest" ),
                       "sTitle: " + sTitle );
            }
        }

        // open another new test project
        String sJimTest2 = FileUtil.concatPath
               ( getTestClassDirectory(), "jimtest2.jiml" );
        FileUtil.delete( sJimTest2 );
        Runnable runTemp = new Runnable() {
            public void run() {
                SwingUtilities.invokeLater
                       (new Runnable() {
                           public void run() {
                               pViewController.open();
                           }
                       } );
            }
        };
        new Thread( runTemp ).start();
        Util.sleep( 3 );
        Window[] apWindow = pViewController.getOwnedWindows();
        // Nr. 10
        bugIf( apWindow.length != 2
               , "apWindow.length: " + apWindow.length );
        bugIf( !(apWindow[ 1 ] instanceof JDialog ) );
        
        JDialog dlgFileChooser = (JDialog)apWindow[ 1 ];
        JTextField txtOpenFile = (JTextField)SwingUtil.getFirstComponent
               ( dlgFileChooser,
                 "javax.swing.plaf.metal.MetalFileChooserUI$1" );
        bugIf( txtOpenFile == null );
        txtOpenFile.setText( sJimTest2 );
        Util.sleep( 1 );
        
        Testable pTestable = new Testable() {
            public boolean test( Object oComponent_ ) {
                if ( oComponent_ instanceof JButton &&
                     ((JButton)oComponent_).getText().equals( "Open" ) )
                {
                    return true;
                }
                
                return false;
            }
        };
        JButton btnOpen = (JButton)SwingUtil.getComponent
               ( dlgFileChooser,
                 pTestable,
                 0 );
        // Nr. 13
        bugIf( btnOpen == null, "Open button is null!" );
        btnOpen.doClick();
        Util.sleep( 2 );
        
        apWindow = pViewController.getOwnedWindows();
        bugIf( apWindow.length != 3 );
        bugIf( !(apWindow[ 2 ] instanceof JDialog ) );
        JDialog dlgMessage = (JDialog)apWindow[ 2 ];
        SwingUtil.pressOKButton( dlgMessage );
        Util.sleep( 1 );
        
        String sTitle = pViewController.getTitle();
        bugIf( !sTitle.equals( "Jim: Jimtest" ),
               "sTitle: " + sTitle );
        
        // new
        pViewController.newFile();
        Util.sleep( 1 );
        
        bugIf( pTextComponent.getDocument().getLength() != 0,
               "You did open a new empty file and still your notes text is not empty." );
        
        sTitle = pViewController.getTitle();
        bugIf( !sTitle.equals( "Jim" ),
               "sTitle should be 'Jim', but is: " + sTitle );
        
        // change something
        _changeSomething( pViewController );
        
        // save as
        {
            String sProjectFile = FileUtil.concatPath
                   ( getTestClassDirectory(), "jimtest2.jiml" );
            FileUtil.delete( sProjectFile );
            runTemp = new Runnable() {
                public void run() {
                    pViewController.saveAs();
                }
            };
            new Thread( runTemp ).start();
            Util.sleep( 3 );
            apWindow = pViewController.getOwnedWindows();
            
            dlgFileChooser = (JDialog)apWindow[ apWindow.length - 1 ];
            JTextField txtSaveFile = (JTextField)SwingUtil.
                   getFirstComponent( dlgFileChooser,
                                      "javax.swing.plaf.metal.MetalFileChooserUI$1" );
            bugIf( txtSaveFile == null );
            txtSaveFile.setText( sProjectFile );
            Util.sleep( 1 );
                
            pTestable = new Testable() {
                public boolean test( Object oComponent_ ) {
                    if ( oComponent_ instanceof JButton &&
                         ((JButton)oComponent_).getText().equals( "Save As" ) )
                    {
                        return true;
                    }
                    
                    return false;
                }
            };
            JButton btnSave = (JButton)SwingUtil.getComponent
                   ( dlgFileChooser,
                     pTestable,
                     0 );
            bugIf( btnSave == null, "Save button is null!" );
            btnSave.doClick();
            Util.sleep( 2 );
            
            sTitle = pViewController.getTitle();
            bugIf( !sTitle.equals( "Jim: Jimtest2" ),
                   "sTitle: " + sTitle );
        }

        // add and delete note and test if enter still works
        {
            bugIf( FocusManager.getCurrentManager() instanceof
                   OKCancelFocusManager );
            
            JButton btnAdd = (JButton)SwingUtil.getFirstComponent
                   ( pViewController,
                     "AddButton" );
            bugIf( btnAdd == null );
            btnAdd.doClick();
            Util.sleep( 1 );
            
            // Nr. 27
            bugIf( !(FocusManager.getCurrentManager() instanceof
                     OKCancelFocusManager),
                   "Focus manager is not instance of OKCancelFocusManager, but should!" );
            
            // get text field
            JTextField txtInput = (JTextField)SwingUtil.getFirstComponent
                   ( pViewController,
                     "InputText" );
            txtInput.setText( "Bla" );
            Util.sleep( 1 );
            
            // push ok
            final JButton btnOK = (JButton)SwingUtil.getFirstComponent
                   ( pViewController,
                     "PanelOKButton" );
            SwingUtilities.invokeLater( new Runnable() {
                    public void run() {
                        btnOK.doClick();
                    }
            } );
            Util.sleep( 3 );

            bugIf( FocusManager.getCurrentManager() instanceof
                   OKCancelFocusManager,
                   "Old focus manager should have been restored, but is not!" );

            // Nr. 29: has title '*' now?
            bugIf( !pViewController.isModified() );
            
            pViewController.save();
            bugIf( pViewController.isModified() );
            
            // select bla note
            JTree pTree = (JTree)SwingUtil.getFirstComponent
                   ( pViewController,
                     "NotesTree" );
            pTree.expandRow( 0 );
            pTree.expandRow( 1 );
            Util.sleep( 2 );
            pTree.setSelectionInterval( 1, 1 );
            Util.sleep( 1 );
            
            // delete
            final JButton btnDelete = (JButton)SwingUtil.getFirstComponent
                   ( pViewController,
                     "DeleteButton" );
            bugIf( btnDelete == null );
            SwingUtilities.invokeLater( new Runnable() {
                public void run() {
                    btnDelete.doClick();
                }
            } );
            Util.sleep( 1 );
            
            // push dialog ok
            apWindow = pViewController.getOwnedWindows();
            dlgFileChooser = (JDialog)apWindow[ apWindow.length - 1 ];
            final JButton btnDialogOK = (JButton)SwingUtil.getFirstComponent
                       ( dlgFileChooser,
                         "PanelOKButton" );
            bugIf( btnDialogOK == null );
            SwingUtilities.invokeLater( new Runnable() {
                public void run() {
                    btnDialogOK.doClick();
                }
            } );
            Util.sleep( 3 );
            bugIf( FocusManager.getCurrentManager() instanceof
                   OKCancelFocusManager,
                   "Focus manager is of type OKCancelFocusManager but shouldnt!" );
            
            // Nr. 34
            bugIf( !pViewController.isModified(),
                   "Note has been deleted, still file is not marked as modified!" );
            
            // enter text and enter key
            JTextArea pTextArea = (JTextArea)SwingUtil.getFirstComponent
                   ( pViewController, "_txtNotes" );
            pTextArea.requestFocus();
            Util.sleep( 2 );
            SwingUtil.dispatchEnterKeyEvent( pTextArea );
            Util.sleep( 2 );
            SwingUtil.dispatchKeyEvent( pTextArea, 'x' );
            Util.sleep( 2 );
            
            // test if it is ok
            
            String sContent = null;
            try {
                sContent = pDocument.getText
                       ( 0, pDocument.getLength() );
            } catch( Exception pException ) {
            }
            
            bugIf( !sContent.equals( "x\nx" ),
                   "Text content should be \"x\nx\" but is: \"" +
                   sContent + "\"" );
        }
        bugIf( FocusManager.getCurrentManager() instanceof
               OKCancelFocusManager,
               "Focus manager is of type OKCancelFocusManager but shouldnt!" );
        
        // Nr. 37   Date bug in todo tasks
        {
            // switch to todo
            {
                JTabbedPane pTabbedPane = (JTabbedPane)SwingUtil.getFirstComponent( pViewController,
                                                                                    "TabbedPane" );
                bugIf( pTabbedPane == null );
                pTabbedPane.setSelectedIndex( 1 );
                Util.sleep( 1 );
                
                // add task
                JTextField txtStart = null;
                JPanel pnlTask = null;
                {
                    final JButton btnNewTask = (JButton)SwingUtil.getFirstComponent( pViewController,
                                                                                     "NewTask" );
                    SwingUtil.doClick( btnNewTask );
                    Util.sleep( 2 );
                    
                    pnlTask = (JPanel)SwingUtil.
                           getFirstComponent( pViewController,
                                              ToDoPanel.
                                              S_TASK_NAME );
                    bugIf( pnlTask == null, "pnlTask is null!" );

                    txtStart = (JTextField)SwingUtil.getFirstComponent
                           ( pnlTask,
                             "StartText" );
                    bugIf( txtStart == null );
                    txtStart.setText( "1999-05-12" );

                    JTextField txtTemp = (JTextField)SwingUtil.getFirstComponent( pnlTask,
                                                                                  "TitleText" );
                    txtTemp.setText( "Just do it!" );

                    OKCancelPanel pnlOKCancel = (OKCancelPanel)SwingUtil.getFirstComponent( pnlTask,
                                                                                            "OKCancelPanel" );
                    bugIf( pnlOKCancel == null );
                    JButton btnOK = pnlOKCancel.getOKButton();
                    SwingUtil.doClick( btnOK );
                    
                    Util.sleep( 2 );
                }
                
                ToDoPanel pnlToDo = (ToDoPanel)SwingUtil.getFirstComponent( pViewController,
                                                                            ToDoPanel.getToDoName() );
                bugIf( pnlToDo == null, "ToDoPanel is null!" );
                ToDoTask pToDoTask = pnlToDo.getToDoTask();
                bugIf( !pToDoTask.getStart().equals( "1999-05-12" ),
                       "ToDoTask has wrong start value '" +
                       pToDoTask.getStart() + 
                       "' instead of '1999-05-12'!" );
                
                // save
                pViewController.save();
                
                // open again
                String sProjectFile = FileUtil.concatPath
                       ( getTestClassDirectory(), "jimtest2.jiml" );
                _open( pViewController, sProjectFile );
                Util.sleep( 1 );
                
                // test start date
                {
                    JList lstToDo = (JList)SwingUtil.getFirstComponent( pViewController,
                                                                        "ToDoList" );
                    lstToDo.setSelectionInterval( 0, 0 );
                    Util.sleep( 1 );
                    final JButton btnEditTask = (JButton)SwingUtil.getFirstComponent( pViewController,
                                                                                      "EditTask" );
                    bugIf( btnEditTask == null, "Couldn't find edit task button!" );
                    SwingUtil.doClick( btnEditTask );
                    Util.sleep( 2 );
                    
                    bugIf( !txtStart.getText().equals( "1999-05-12" ),
                           "Start value is '" + txtStart.getText() +
                           "' but should be '1999-05-12'" );

                    _clickOKButton( pnlTask );
                }
            }

            Util.sleep( 1 );
        }

        // DateFields are not cleared bug.
        // Edit task and finished field, then add new task
        // and check for empty finished field.
        {
            // edit finished date
            {
                JTextField txtFinished = null;
                
                final JButton btnEditTask = (JButton)SwingUtil.getFirstComponent( pViewController,
                                                                                  "EditTask" );
                SwingUtil.doClick( btnEditTask );
                Util.sleep( 2 );
                
                JPanel pnlTask = (JPanel)SwingUtil.
                       getFirstComponent( pViewController,
                                          ToDoPanel.
                                          S_TASK_NAME );
                bugIf( pnlTask == null, "2. pnlTask is null!" );
                
                txtFinished = (JTextField)SwingUtil.getFirstComponent
                       ( pnlTask,
                         "FinishedText" );
                bugIf( txtFinished == null, "txtFinished is null!" );
                bugIf( !Util.isEmpty( txtFinished.getText() ) );
                txtFinished.setText( "1999-05-12" );
             
                //
                // click ok button
                //
                OKCancelPanel pnlOKCancel = (OKCancelPanel)SwingUtil.getFirstComponent( pnlTask,
                                                                                        "OKCancelPanel" );
                bugIf( pnlOKCancel == null );
                JButton btnOK = pnlOKCancel.getOKButton();
                SwingUtil.doClick( btnOK );
                
                Util.sleep( 2 );
            }

            // check finished date in new task
            {
                JTextField txtFinished = null;
                
                final JButton btnNewTask = (JButton)SwingUtil.getFirstComponent( pViewController,
                                                                                  "NewTask" );
                SwingUtil.doClick( btnNewTask );
                Util.sleep( 2 );
                
                JPanel pnlTask = (JPanel)SwingUtil.
                       getFirstComponent( pViewController,
                                          ToDoPanel.
                                          S_TASK_NAME );
                bugIf( pnlTask == null, "3. pnlTask is null!" );
                
                txtFinished = (JTextField)SwingUtil.getFirstComponent
                       ( pnlTask,
                         "FinishedText" );
                bugIf( txtFinished == null );
                bugIf( !Util.isEmpty( txtFinished.getText() ), 
                       "Finished date is not empty!" );
            }
        }

        // test tree loading for todo projects
        {
            // load jim/test/test1.jiml
            String sOpenFile = FileUtil.concatPath
                   ( getTestClassDirectory(),
                     "test1.jiml" );
            pViewController.open( sOpenFile );
            
            // get todos tree model
            DefaultMutableTreeNode tnRoot = null;
            {
                // switch to todo panel
                _selectToDoPanel( pViewController );

                JTree pTree = (JTree)SwingUtil.getFirstComponent( pViewController,
                                                                  "ToDoTree" );
                bugIf( pTree == null );
                
                // get root
                tnRoot = (DefaultMutableTreeNode)pTree.getModel().getRoot();                
            }

            // check how many children there are
            bugIf( tnRoot.getChildCount() != 1,
                   "There should be one child in todo project but there are: " +
                   tnRoot.getChildCount() );
        }
    }
    
    public static void main(String[] asArg_) {
        Test pTest = (Test)(new MainTest());
        pTest.setVerbose( true );
        pTest.run();
        pTest.printResult(pTest);

        System.exit(0);
    }
}
